"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pullContent = exports.resetLastChangedFiles = exports.resetFiles = exports.getChangedFiles = exports.getChangedContentFiles = exports.isContentFile = void 0;
const path_1 = __importDefault(require("path"));
const config_js_1 = require("./config.js");
const FILE_STATUS = {
    ADDED: 'A',
    RENAMED: 'R',
    MODIFIED: 'M',
    DELETED: 'D',
};
const isContentFile = (filePath) => {
    const ext = path_1.default.extname(filePath).slice(1);
    return config_js_1.CONTENT_FILE_EXT.has(ext);
};
exports.isContentFile = isContentFile;
const getChangedContentFiles = async ({ run }) => {
    const changedFiles = await (0, exports.getChangedFiles)({ run });
    return changedFiles.filter((file) => (0, exports.isContentFile)(file.filePath));
};
exports.getChangedContentFiles = getChangedContentFiles;
const throwIfError = ({ stderr, stdout, exitCode }, message) => {
    if (exitCode !== 0) {
        throw new Error(stderr || stdout || message);
    }
};
const getChangedFiles = async ({ run }) => {
    const fetchResult = await run('git', ['fetch'], { stdout: 'pipe' });
    throwIfError(fetchResult, 'Cant fetch');
    const diffResult = await run('git', ['diff', '--name-status', 'HEAD...FETCH_HEAD'], { stdout: 'pipe' });
    throwIfError(diffResult, 'Cant make diff');
    const changedFiles = diffResult.stdout
        .split('\n')
        .filter(Boolean)
        .map((str) => {
        const [[status], filePath, newFilePath] = str.split('\t');
        return {
            filePath,
            newFilePath,
            status,
        };
    });
    return changedFiles;
};
exports.getChangedFiles = getChangedFiles;
const resetFiles = async ({ run, changedFiles }) => {
    // remove added and renamed
    const filesForDeletion = changedFiles.flatMap((file) => {
        if (file.status === FILE_STATUS.RENAMED) {
            return [file.newFilePath];
        }
        if (file.status === FILE_STATUS.ADDED) {
            return [file.filePath];
        }
        return [];
    });
    if (filesForDeletion.length !== 0) {
        const rmResult = await run('rm', filesForDeletion, { stdout: 'pipe' });
        throwIfError(rmResult, 'Cant checkout current version of changed files');
    }
    // restore modified and deleted
    const filesForCheckout = changedFiles.flatMap((file) => {
        if ([FILE_STATUS.MODIFIED, FILE_STATUS.DELETED, FILE_STATUS.RENAMED].includes(file.status)) {
            return [file.filePath];
        }
        return [];
    });
    if (filesForCheckout.length !== 0) {
        const checkoutResult = await run('git', ['checkout', 'HEAD', '--', ...filesForCheckout], { stdout: 'pipe' });
        throwIfError(checkoutResult, 'Cant checkout current version of changed files');
    }
};
exports.resetFiles = resetFiles;
const resetLastChangedFiles = () => {
    lastChangedFiles = undefined;
};
exports.resetLastChangedFiles = resetLastChangedFiles;
let lastChangedFiles;
const pullContent = async ({ run }) => {
    if (lastChangedFiles) {
        await (0, exports.resetFiles)({ run, changedFiles: lastChangedFiles });
    }
    const changedFiles = await (0, exports.getChangedContentFiles)({ run });
    if (changedFiles.length === 0) {
        return changedFiles;
    }
    // added, modified and renamed - checkout
    const filesForCheckout = changedFiles.flatMap((file) => {
        const matches = [FILE_STATUS.ADDED, FILE_STATUS.MODIFIED, FILE_STATUS.RENAMED].includes(file.status);
        if (!matches) {
            return [];
        }
        if (file.status === FILE_STATUS.RENAMED) {
            return [file.newFilePath];
        }
        return [file.filePath];
    });
    if (filesForCheckout.length !== 0) {
        const checkoutResult = await run('git', ['checkout', 'FETCH_HEAD', '--', ...filesForCheckout], { stdout: 'pipe' });
        throwIfError(checkoutResult, 'Cant checkout latest version of changed files');
    }
    // deleted and renamed - remove
    const filesForDeletion = changedFiles.flatMap((file) => {
        if ([FILE_STATUS.RENAMED, FILE_STATUS.DELETED].includes(file.status)) {
            return [file.filePath];
        }
        return [];
    });
    if (filesForDeletion.length !== 0) {
        const rmResult = await run('rm', filesForDeletion, { stdout: 'pipe' });
        throwIfError(rmResult, 'Cant remove deleted files');
    }
    lastChangedFiles = changedFiles;
    return changedFiles;
};
exports.pullContent = pullContent;
